@@ -1,14 +1,15 @@
 import gc
 
 from copy import deepcopy
-from itertools import groupby
+from itertools import groupby, islice
 from collections import defaultdict
 from typing import Iterable, Sequence, Any, Optional, Tuple, Union
 
 from coba.learners import Learner
 from coba.contexts import CobaContext
-from coba.pipes import Source, Filter, SourceFilters
-from coba.environments import SimulatedEnvironment, FilteredEnvironment
+from coba.pipes import Source, Filter
+from coba.pipes.core import SourceFilters
+from coba.environments import SimulatedEnvironment
 
 from coba.experiments.tasks import LearnerTask, EnvironmentTask, EvaluationTask
 from coba.experiments.results import Result
@@ -19,19 +20,19 @@ class WorkItem:
         environ_id: Optional[int],
         learner_id: Optional[int],
         environ   : Optional[SimulatedEnvironment],
-        learner   : Optional[Learner],        
+        learner   : Optional[Learner],
         task      : Union[LearnerTask, EnvironmentTask, EvaluationTask]) -> None:
 
         self.learner_id = learner_id
         self.environ_id = environ_id
-        self.learner = learner
-        self.environ = environ
-        self.task    = task
+        self.learner    = learner
+        self.environ    = environ
+        self.task       = task
 
 class CreateWorkItems(Source[Iterable[WorkItem]]):
 
-    def __init__(self, 
-        environs        : Sequence[SimulatedEnvironment], 
+    def __init__(self,
+        environs        : Sequence[SimulatedEnvironment],
         learners        : Sequence[Learner],
         learner_task    : LearnerTask,
         environment_task: EnvironmentTask,
@@ -42,7 +43,7 @@ class CreateWorkItems(Source[Iterable[WorkItem]]):
 
         self._learner_task     = learner_task
         self._environment_task = environment_task
-        self._evaluation_task  = evaluation_task        
+        self._evaluation_task  = evaluation_task
 
     def read(self) -> Iterable[WorkItem]:
 
@@ -84,7 +85,7 @@ class RemoveFinished(Filter[Iterable[WorkItem], Iterable[WorkItem]]):
 
 class ChunkBySource(Filter[Iterable[WorkItem], Iterable[Sequence[WorkItem]]]):
 
-    def filter(self, items: Iterable[WorkItem]) -> Iterable[Iterable[WorkItem]]:
+    def filter(self, items: Iterable[WorkItem]) -> Iterable[Sequence[WorkItem]]:
 
         items  = list(items)
         chunks = defaultdict(list)
@@ -102,23 +103,29 @@ class ChunkBySource(Filter[Iterable[WorkItem], Iterable[Sequence[WorkItem]]]):
             yield list(sorted(chunk, key=lambda c: (c.environ_id, -1 if c.learner_id is None else c.learner_id)))
 
     def _get_source(self, env):
-        return env._source if isinstance(env, (FilteredEnvironment, SourceFilters)) else env
+        return env._source if isinstance(env, SourceFilters) else env
 
-class ChunkByTask(Filter[Iterable[WorkItem], Iterable[Iterable[WorkItem]]]):
+class ChunkByTask(Filter[Iterable[WorkItem], Iterable[Sequence[WorkItem]]]):
 
-    def filter(self, workitems: Iterable[WorkItem]) -> Iterable[Iterable[WorkItem]]:
+    def filter(self, workitems: Iterable[WorkItem]) -> Iterable[Sequence[WorkItem]]:
 
-        workitems = list(workitems)
+        #this makes sure all items are in order by source
+        for chunk in ChunkBySource().filter(workitems):
+            for workitem in chunk:
+                yield [ workitem ]
 
-        learner_items = [w for w in workitems if w.environ_id is None]
-        environ_items = [w for w in workitems if w.learner_id is None]
-        eval_items    = [w for w in workitems if w.environ_id is not None and w.learner_id is not None]
+class MaxChunkSize(Filter[Iterable[Sequence[WorkItem]], Iterable[Sequence[WorkItem]]]):
+    def __init__(self, max_tasks) -> None:
+        self._max_tasks = max_tasks
 
-        for item in sorted(learner_items, key = lambda t: t.learner_id):
-            yield [ item ]
+    def filter(self, chunks: Iterable[Sequence[WorkItem]]) -> Iterable[Sequence[WorkItem]]:
 
-        for item in sorted(environ_items+eval_items, key=lambda t:( t.environ_id, t.learner_id if t.learner_id is not None else -1)):
-            yield [ item ]
+        for chunk in chunks:
+            chunk = iter(chunk)
+            max_task_chunk = list(islice(chunk,self._max_tasks or None))
+            while max_task_chunk:
+                yield max_task_chunk
+                max_task_chunk = list(islice(chunk,self._max_tasks or None))
 
 class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
 
@@ -165,7 +172,7 @@ class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
                                 gc.collect()
 
                             if not interactions:
-                                CobaContext.logger.log(f"Environment {env_id} has nothing to evaluate (this is often due to `take` being larger than source).")
+                                CobaContext.logger.log(f"Environment {env_id} has nothing to evaluate (this is likely due to having too few interactions).")
                                 break
 
                         for workitem in work_for_env_filter:
@@ -195,19 +202,19 @@ class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
     def _get_source(self, task:WorkItem) -> SimulatedEnvironment:
         if task.environ is None:
             return None
-        elif isinstance(task.environ, (SourceFilters, FilteredEnvironment)):
-            return task.environ._source 
+        elif isinstance(task.environ, SourceFilters):
+            return task.environ._source
         else:
             return task.environ
-    
+
     def _get_source_sort(self, task:WorkItem) -> int:
         return self._source_id.get(id(self._get_source(task)),-1)
 
     def _get_id_filter(self, task:WorkItem) -> Tuple[int, Filter[SimulatedEnvironment,SimulatedEnvironment]]:
         if task.environ is None:
             return (-1,None)
-        elif isinstance(task.environ, (SourceFilters, FilteredEnvironment)):
-            return (task.environ_id, task.environ._filter) 
+        elif isinstance(task.environ, SourceFilters):
+            return (task.environ_id, task.environ._filter)
         else:
             return (task.environ_id, None)
 
