@@ -217,30 +217,36 @@ class VowpalArgsLearner(Learner):
         self._n_actions = None
         self._actions   = None
 
-        if "--cb " in args or "--cb_explore " in args:
-            self._n_actions = int(re.match("--cb.*?\s+(\d*)\s*-?.*$", args).group(1))
+        if not self._adf:
+            n_action_str = re.match("--cb.*?\s*(\d*)\\b.*", args).group(1)
+            self._n_actions = int(n_action_str) if n_action_str else None
+            #useful for removing the n_actions from args 
+            #re.sub("(--cb)\s*(\d+)", '\g<1>', "--cb 123", count=1)
 
         self._vw = vw or VowpalMediator()
 
-        if self._adf or self._n_actions is not None:
-            self._vw.init_learner(args, 4)
-
     @property
     def params(self) -> Dict[str, Any]:
         return {"family": "vw", 'args': self._args.replace("--quiet","").strip()}
 
     def predict(self, context: Context, actions: Sequence[Action]) -> Tuple[Probs, Info]:
 
-        if not self._adf and not self._actions:
-            self._actions = actions
+        if not self._vw.is_initialized and self._adf:
+            self._vw.init_learner(self._args, 4)
 
-        if not self._vw.is_initialized: #this should only be true for not adf with no actions given
+        if not self._vw.is_initialized and not self._adf and self._n_actions:
+            self._vw.init_learner(self._args, 4)
+
+        if not self._vw.is_initialized and not self._adf and not self._n_actions:
             self._n_actions = len(actions)
             args            = self._args.replace('--cb_explore','').replace('--cb','')
             args            = f"--cb_explore {len(actions)} " if self._explore else f"--cb {len(actions)} " + args
             args            = args.strip()
             self._vw.init_learner(args,4)
 
+        if not self._adf and not self._actions:
+            self._actions = actions
+
         if not self._adf and actions != self._actions:
             raise CobaException("Actions are only allowed to change between predictions when using `adf`.")
 
@@ -273,8 +279,14 @@ class VowpalArgsLearner(Learner):
 
     def learn(self, context: Context, action: Action, reward: float, probability: float, info: Info) -> None:
 
-        if not self._vw.is_initialized:
-            raise CobaException("When using `cb  without `adf` predict must be called before learn to initialize the vw learner")
+        if not self._vw.is_initialized and self._adf:
+            self._vw.init_learner(self._args, 4)
+
+        if not self._vw.is_initialized and not self._adf and self._n_actions:
+            self._vw.init_learner(self._args, 4)
+
+        if not self._vw.is_initialized and not self._adf and not self._n_actions:
+            raise CobaException("When using `cb` without `adf` predict must be called before learn to initialize the vw learner")
 
         actions = info
         labels  = self._labels(actions, action, reward, probability)
@@ -294,9 +306,6 @@ class VowpalArgsLearner(Learner):
     def _flat(self,features:Any) -> Any:
         return list(Flatten().filter([features]))[0]
 
-    def __reduce__(self):
-        return (type(self), (self._args, self._vw) )
-
 class VowpalEpsilonLearner(VowpalArgsLearner):
     """A wrapper around VowpalArgsLearner that provides more documentation. For more
         information on the types of exploration algorithms availabe in VW see `here`__.
@@ -321,7 +330,9 @@ class VowpalEpsilonLearner(VowpalArgsLearner):
         noconstant    = sum([f for f in features if isinstance(f,(int,float))]) == 0
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs))
+        vw            = kwargs.pop('vw',None)
+
+        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
 class VowpalSoftmaxLearner(VowpalArgsLearner):
     """A wrapper around VowpalArgsLearner that provides more documentation. For more
@@ -351,7 +362,8 @@ class VowpalSoftmaxLearner(VowpalArgsLearner):
         noconstant    = sum([f for f in features if isinstance(f,(int,float))]) == 0
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs))
+        vw            = kwargs.pop('vw',None)
+        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
 class VowpalBagLearner(VowpalArgsLearner):
     """A wrapper around VowpalArgsLearner that provides more documentation. For more
@@ -379,7 +391,8 @@ class VowpalBagLearner(VowpalArgsLearner):
         noconstant    = sum([f for f in features if isinstance(f,(int,float))]) == 0
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs))
+        vw            = kwargs.pop('vw',None)
+        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
 class VowpalCoverLearner(VowpalArgsLearner):
     """A wrapper around VowpalArgsLearner that provides more documentation. For more
@@ -412,7 +425,9 @@ class VowpalCoverLearner(VowpalArgsLearner):
         noconstant    = sum([f for f in features if isinstance(f,(int,float))]) == 0
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs))
+        vw            = kwargs.pop('vw',None)
+
+        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
 class VowpalRegcbLearner(VowpalArgsLearner):
     """A wrapper around VowpalArgsLearner that provides more documentation. For more
@@ -445,7 +460,8 @@ class VowpalRegcbLearner(VowpalArgsLearner):
         noconstant    = sum([f for f in features if isinstance(f,(int,float))]) == 0
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs))
+        vw            = kwargs.pop('vw',None)
+        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
 class VowpalSquarecbLearner(VowpalArgsLearner):
     """A wrapper around VowpalArgsLearner that provides more documentation. For more
@@ -487,7 +503,8 @@ class VowpalSquarecbLearner(VowpalArgsLearner):
         noconstant    = sum([f for f in features if isinstance(f,(int,float))]) == 0
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs))
+        vw            = kwargs.pop('vw',None)
+        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
 
 class VowpalOffPolicyLearner(VowpalArgsLearner):
     """A wrapper around VowpalArgsLearner that provides more documentation. For more
@@ -515,4 +532,5 @@ class VowpalOffPolicyLearner(VowpalArgsLearner):
         noconstant    = sum([f for f in features if isinstance(f,(int,float))]) == 0
         ignore_linear = set(['x','a'])-set(features)
         interactions  = [f for f in features if isinstance(f,str) and len(f) > 1]
-        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs))
+        vw            = kwargs.pop('vw',None)
+        super().__init__(VowpalArgsLearner.make_args(options, noconstant, interactions, ignore_linear, seed, **kwargs), vw)
